import sys
import numpy as np


def finite_difference(fun, x, param):
    """Calculates the Jacobian of function 'fun' in 'x' using finite differences
    input:
      * fun:    function handle, taking x as first argument, and param as second
      * x:      evaluate jacobian at x
      * param:  parameters of fun
    output
      * F: function value evaluated at x
      * J: Jacobian of fun evaluated at x
    """

    # Definine the step length as square root of the float accuracy
    h = np.sqrt(sys.float_info.epsilon)

    F = fun(x, param)

    x_dim = np.size(x)
    J = np.zeros((np.size(F), x_dim))

    # In each loop we obtain one column of the Jacobian
    # This is the seed
    p = np.zeros(x_dim)
    for k in range(x_dim):
        p[k] = 1 * h
        J[:, k] = (fun(x+p, param) - F) / h
        p[k] = 0
    return F, J
